import time
from typing import Optional

import aiohttp
from loguru import logger

from huibiao_framework.client.abstract_client import HuibiaoAbstractClient
from huibiao_framework.client.data_model.model_resp_vo import ModelBaseRespVo
from huibiao_framework.client.data_model.ocr.image_ocr import ImageOcrVo
from huibiao_framework.config import ClientConfig
from huibiao_framework.utils.image import ImageUtils


class ImageOcrClient(HuibiaoAbstractClient):
    """
    图片版面解析接口
    url: http://host:port/image_ocr
    request:
        payload = {"data": encode_base64_string(file_path)}
        headers = {
                "Content-Type": "application/json",  # 根据需要设置内容类型
                "x-request-id": reqid
                }
    response:
        {
        "code": 0,
        "result": {
            "version": "0.3.32",
            "width": 991,
            "height": 647,
            "angle": 0,
            "lines": [
                {
                    "bbox": [
                                [
                                    293.0,
                                    179.0
                                ],
                                [
                                    714.0,
                                    179.0
                                ],
                                [
                                    714.0,
                                    226.0
                                ],
                                [
                                    293.0,
                                    226.0
                                ]
                            ],
                    "text": "学士学位证书",
                    "score": 1.0
                },...
                ]
            }
        }
    """

    def __init__(self, session: Optional[aiohttp.ClientSession] = None):
        super().__init__(client_name="ImageOcr", session=session)

    async def query_by_image_base64(
        self, image_base64: str, reqid: str = "", session_id: str = ""
    ) -> ImageOcrVo:
        reqid = reqid if reqid is not None else ""
        session_id = session_id if session_id is not None else reqid
        session_tag = self.session_tag(session_id)

        payload = {"data": image_base64}
        headers = {
            "Content-Type": "application/json",  # 根据需要设置内容类型
            "x-request-id": reqid,
        }
        start_time = time.time()
        try:
            async with self.session.post(
                ClientConfig.IMAGE_OCR_TYY_URL, json=payload, headers=headers
            ) as resp:
                sp_time = time.time() - start_time
                logger.debug(
                    f"{session_tag},resp-{resp.status}, 响应时间: {sp_time:.2f}秒"
                )
                resp.raise_for_status()
                response_data = await resp.json()
                response_data = ModelBaseRespVo[ImageOcrVo](**response_data)
                return response_data.result
        except aiohttp.ClientError as e:
            logger.error(f"{session_tag}请求异常", e)
            raise e

    async def query_by_image_path(
        self, image_path: str, reqid: str = "", session_id: str = ""
    ) -> ImageOcrVo:
        image_base64 = await ImageUtils.encode_base64(image_path)
        return await self.query_by_image_base64(
            image_base64=image_base64, reqid=reqid, session_id=session_id
        )



from pydantic import BaseModel, Field
from typing import Generic, TypeVar, Optional

from huibiao_framework.backend import CommonStatusCode
from huibiao_framework.backend.status_code import BasicStatusCode

T = TypeVar("T")


class BaseRespVo(BaseModel, Generic[T]):
    code: int
    message: str
    result: Optional[T] = Field(None, description="结果")

    @classmethod
    def success(cls, result: Optional[T]):
        return cls(
            code=CommonStatusCode.SUCCESS.code,
            result=result,
            message=CommonStatusCode.SUCCESS.msg,
        )

    @classmethod
    def from_status_code(cls, http_status: BasicStatusCode, **kwargs):
        return cls(
            code=http_status.code,
            message=http_status.msg.format(**kwargs),
            result=None,
        )



from typing import Dict
from loguru import logger
from prometheus_client import Counter, Gauge


class RequestOperationMetrics:
    """
    算法接口中算法操作的耗时统计
    """

    __METRICS_MAX: Dict[str, float] = dict()
    __METRICS_SUM: Dict[str, float] = dict()
    __METRICS_NUM: Dict[str, int] = dict()
    __AVG_METRICS: Dict[str, float] = dict()
    __ERROR_METRICS: Dict[str, int] = dict()

    _max_metrics = Gauge(
        "operation_max_duration_seconds",
        "The maximum duration of an operation.",
        ["operation"],
    )
    _num_metrics = Counter(
        "operation_count", "The total number of operations.", ["operation"]
    )
    _avg_metrics = Gauge(
        "operation_avg_duration_seconds",
        "The average duration of an operation.",
        ["operation"],
    )
    _error_metrics = Counter(
        "operation_errors_total",
        "The total number of errors in operations.",
        ["operation"],
    )

    @classmethod
    def add(cls, key, cost):
        _max = cls.__METRICS_MAX.get(key, 0)
        _sum = cls.__METRICS_SUM.get(key, 0)
        _num = cls.__METRICS_NUM.get(key, 0)

        new_max = max(_max, cost)
        new_sum = _sum + cost
        new_num = _num + 1
        cls.__METRICS_MAX[key] = new_max
        cls.__METRICS_SUM[key] = new_sum
        cls.__METRICS_NUM[key] = new_num
        cls.__AVG_METRICS[key] = new_sum / new_num
        logger.info(
            f"OpTimeCost：{key}: max={new_max}, sum={new_sum}, num={new_num}, avg={new_sum / new_num}s"
        )
        # 更新 Prometheus 指标
        cls._max_metrics.labels(operation=key).set(new_max)
        cls._num_metrics.labels(operation=key).inc(1)
        cls._avg_metrics.labels(operation=key).set(new_sum / new_num)

    @classmethod
    def add_error(cls, key):
        cls.__ERROR_METRICS[key] = cls.__ERROR_METRICS.get(key, 0) + 1
        logger.info(f"OpErrorNum：{key}: {cls.__ERROR_METRICS[key]}")
        cls._error_metrics.labels(operation=key).inc(1)



from huibiao_framework.backend.status_code import BasicStatusCode
from typing import TypeVar, Generic

class BasicException(Exception):
    """
    基础异常类
    """

    def __init__(self, code: int, msg: str):
        super().__init__(msg)
        self.code = code
        self.msg = msg


# 定义泛型变量 T，限定为 BasicStatusCode 的子类
T = TypeVar('T', bound=BasicStatusCode)

class BasicCommonException(BasicException, Generic[T]):
    def __init__(self, status_code: T, **kwargs):
        super().__init__(code=status_code.code, msg=status_code.msg.format(**kwargs))


from .status_code import BasicStatusCode, CommonStatusCode
from .exception import BasicException, BasicCommonException
from .vo import BaseRespVo
from .handler import timing_and_exception_handler
from .log_setting import LoggerSetup

__all__ = [
    # 响应码
    "BasicStatusCode",
    "CommonStatusCode",
    # 异常类
    "BasicException",
    "BasicCommonException",
    # 响应类
    "BaseRespVo",
    # 装饰器
    "timing_and_exception_handler",
    # 日志
    "LoggerSetup",
]



from abc import ABC
from typing import Optional

import aiohttp


class HuibiaoAbstractClient(ABC):
    def __init__(
        self, client_name: str, *, session: Optional[aiohttp.ClientSession] = None
    ):
        self.__session = session
        self.__name = client_name
        assert self.__session is not None, "会话不能为空"

    @property
    def session(self):
        return self.__session

    @property
    def client_name(self) -> Optional[str]:
        return self.__name

    def session_tag(self, session_id: str) -> str:
        session_id_suffix = f"[{session_id}]" if session_id else ""
        return f"[{self.client_name}]{session_id_suffix}"
